@@ -45,6 +45,7 @@
 #include <pcl/pcl_base.h>
 #include <pcl/PointIndices.h>
 #include <pcl/conversions.h>
+#include <pcl/exceptions.h>
 #include <locale>
 
 namespace pcl
@@ -223,6 +224,24 @@ namespace pcl
     }
   }
 
+  typedef enum
+  {
+    BORDER_CONSTANT = 0, BORDER_REPLICATE = 1,
+    BORDER_REFLECT = 2, BORDER_WRAP = 3,
+    BORDER_REFLECT_101 = 4, BORDER_TRANSPARENT = 5,
+    BORDER_DEFAULT = BORDER_REFLECT_101
+  } InterpolationType;
+
+  /** \brief \return the right index according to the interpolation type.
+    * \note this is adapted from OpenCV
+    * \param p the index of point to interpolate
+    * \param length the top/bottom row or left/right column index
+    * \param type the requested interpolation
+    * \throws pcl::BadArgumentException if type is unknown
+    */
+  PCL_EXPORTS int
+  interpolatePointIndex (int p, int length, InterpolationType type);
+
   /** \brief Concatenate two pcl::PCLPointCloud2.
     * \param[in] cloud1 the first input point cloud dataset
     * \param[in] cloud2 the second input point cloud dataset
@@ -380,6 +399,31 @@ namespace pcl
                   const std::vector<pcl::PointIndices> &indices, 
                   pcl::PointCloud<PointOutT> &cloud_out);
 
+  /** \brief Copy a point cloud inside a larger one interpolating borders.
+    * \param[in] cloud_in the input point cloud dataset
+    * \param[out] cloud_out the resultant output point cloud dataset
+    * \param top
+    * \param bottom
+    * \param left
+    * \param right
+    * Position of cloud_in inside cloud_out is given by \a top, \a left, \a bottom \a right.
+    * \param[in] border_type the interpolating method (pcl::BORDER_XXX)
+    *  BORDER_REPLICATE:     aaaaaa|abcdefgh|hhhhhhh
+    *  BORDER_REFLECT:       fedcba|abcdefgh|hgfedcb
+    *  BORDER_REFLECT_101:   gfedcb|abcdefgh|gfedcba
+    *  BORDER_WRAP:          cdefgh|abcdefgh|abcdefg
+    *  BORDER_CONSTANT:      iiiiii|abcdefgh|iiiiiii  with some specified 'i'
+    *  BORDER_TRANSPARENT:   mnopqr|abcdefgh|tuvwxyz  where m-r and t-z are orignal values of cloud_out
+    * \param value
+    * \throw pcl::BadArgumentException if any of top, bottom, left or right is negative.
+    * \ingroup common
+    */
+  template <typename PointT> void
+  copyPointCloud (const pcl::PointCloud<PointT> &cloud_in,
+                  pcl::PointCloud<PointT> &cloud_out,
+                  int top, int bottom, int left, int right,
+                  pcl::InterpolationType border_type, const PointT& value);
+
   /** \brief Concatenate two datasets representing different fields.
     *
     * \note If the input datasets have overlapping fields (i.e., both contain
