@@ -42,6 +42,9 @@
 #include <pcl/pcl_macros.h>
 #include <bitset>
 #include <pcl/register_point_struct.h>
+#include <boost/mpl/contains.hpp>
+#include <boost/mpl/fold.hpp>
+#include <boost/mpl/vector.hpp>
 
 /**
   * \file pcl/point_types.h
@@ -227,6 +230,11 @@ namespace pcl
     */
   struct PPFSignature;
 
+  /** \brief Members: float f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, alpha_m
+    * \ingroup common
+    */
+  struct CPPFSignature;
+
   /** \brief Members: float f1, f2, f3, f4, r_ratio, g_ratio, b_ratio, alpha_m
     * \ingroup common
     */
@@ -513,6 +521,20 @@ POINT_CLOUD_REGISTER_POINT_STRUCT (pcl::PPFSignature,
     (float, alpha_m, alpha_m)
 )
 
+POINT_CLOUD_REGISTER_POINT_STRUCT (pcl::CPPFSignature,
+    (float, f1, f1)
+    (float, f2, f2)
+    (float, f3, f3)
+    (float, f4, f4)
+    (float, f5, f5)
+    (float, f6, f6)
+    (float, f7, f7)
+    (float, f8, f8)
+    (float, f9, f9)
+    (float, f10, f10)
+    (float, alpha_m, alpha_m)
+)
+
 POINT_CLOUD_REGISTER_POINT_STRUCT (pcl::PPFRGBSignature,
     (float, f1, f1)
     (float, f2, f2)
@@ -636,6 +658,84 @@ namespace pcl
       }
     }
   };
+
+  namespace traits
+  {
+
+    /** \brief Metafunction to check if a given point type has a given field.
+     *
+     *  Example usage at run-time:
+     *
+     *  \code
+     *  bool curvature_available = pcl::traits::has_field<PointT, pcl::fields::curvature>::value;
+     *  \endcode
+     *
+     *  Example usage at compile-time:
+     *
+     *  \code
+     *  BOOST_MPL_ASSERT_MSG ((pcl::traits::has_field<PointT, pcl::fields::label>::value),
+     *                        POINT_TYPE_SHOULD_HAVE_LABEL_FIELD,
+     *                        (PointT));
+     *  \endcode
+     */
+    template <typename PointT, typename Field>
+    struct has_field : boost::mpl::contains<typename pcl::traits::fieldList<PointT>::type, Field>::type
+    { };
+
+    /** Metafunction to check if a given point type has all given fields. */
+    template <typename PointT, typename Field>
+    struct has_all_fields : boost::mpl::fold<Field,
+                                             boost::mpl::bool_<true>,
+                                             boost::mpl::and_<boost::mpl::_1,
+                                                              has_field<PointT, boost::mpl::_2> > >::type
+    { };
+
+    /** Metafunction to check if a given point type has any of the given fields. */
+    template <typename PointT, typename Field>
+    struct has_any_field : boost::mpl::fold<Field,
+                                            boost::mpl::bool_<false>,
+                                            boost::mpl::or_<boost::mpl::_1,
+                                                            has_field<PointT, boost::mpl::_2> > >::type
+    { };
+
+    /** Metafunction to check if a given point type has x, y, and z fields. */
+    template <typename PointT>
+    struct has_xyz : has_all_fields<PointT, boost::mpl::vector<pcl::fields::x,
+                                                               pcl::fields::y,
+                                                               pcl::fields::z> >
+    { };
+
+    /** Metafunction to check if a given point type has normal_x, normal_y, and
+      * normal_z fields. */
+    template <typename PointT>
+    struct has_normal : has_all_fields<PointT, boost::mpl::vector<pcl::fields::normal_x,
+                                                                  pcl::fields::normal_y,
+                                                                  pcl::fields::normal_z> >
+    { };
+
+    /** Metafunction to check if a given point type has curvature field. */
+    template <typename PointT>
+    struct has_curvature : has_field<PointT, pcl::fields::curvature>
+    { };
+
+    /** Metafunction to check if a given point type has intensity field. */
+    template <typename PointT>
+    struct has_intensity : has_field<PointT, pcl::fields::intensity>
+    { };
+
+    /** Metafunction to check if a given point type has either rgb or rgba field. */
+    template <typename PointT>
+    struct has_color : has_any_field<PointT, boost::mpl::vector<pcl::fields::rgb,
+                                                                pcl::fields::rgba> >
+    { };
+
+    /** Metafunction to check if a given point type has label field. */
+    template <typename PointT>
+    struct has_label : has_field<PointT, pcl::fields::label>
+    { };
+
+  }
+
 } // namespace pcl
 
 #if defined _MSC_VER
