@@ -38,6 +38,7 @@
 #include <pcl/io/vtk_lib_io.h>
 #include <pcl/io/impl/vtk_lib_io.hpp>
 #include <pcl/PCLPointCloud2.h>
+#include <vtkVersion.h>
 #include <vtkCellArray.h>
 #include <vtkCellData.h>
 #include <vtkDoubleArray.h>
@@ -80,18 +81,18 @@ pcl::io::savePolygonFile (const std::string &file_name, const pcl::PolygonMesh&
   // TODO: what about sensor position and orientation?!?!?!?
   // TODO: how to adequately catch exceptions thrown by the vtk writers?!
   std::string extension = file_name.substr (file_name.find_last_of (".") + 1);
-  if (extension == ".pcd") // no Polygon, but only a point cloud
+  if (extension == "pcd") // no Polygon, but only a point cloud
   {
     int error_code = pcl::io::savePCDFile (file_name, mesh.cloud);
     if (error_code != 0)
       return (0);
     return (static_cast<int> (mesh.cloud.width * mesh.cloud.height));
   }
-  else if (extension == ".vtk")
+  else if (extension == "vtk")
    return (pcl::io::savePolygonFileVTK (file_name, mesh));
-  else if (extension == ".ply")
+  else if (extension == "ply")
    return (pcl::io::savePolygonFilePLY (file_name, mesh));
-  else if (extension == ".stl" )
+  else if (extension == "stl" )
     return (pcl::io::savePolygonFileSTL (file_name, mesh));
   else
   {
@@ -178,7 +179,11 @@ pcl::io::savePolygonFileVTK (const std::string &file_name, const pcl::PolygonMes
   pcl::io::mesh2vtk (mesh, poly_data);
 
   vtkSmartPointer<vtkPolyDataWriter> poly_writer = vtkSmartPointer<vtkPolyDataWriter>::New ();
+#if VTK_MAJOR_VERSION < 6
   poly_writer->SetInput (poly_data);
+#else
+  poly_writer->SetInputData (poly_data);
+#endif
   poly_writer->SetFileName (file_name.c_str ());
   poly_writer->Write ();
 
@@ -194,7 +199,11 @@ pcl::io::savePolygonFilePLY (const std::string &file_name, const pcl::PolygonMes
   pcl::io::mesh2vtk (mesh, poly_data);
 
   vtkSmartPointer<vtkPLYWriter> poly_writer = vtkSmartPointer<vtkPLYWriter>::New ();
+#if VTK_MAJOR_VERSION < 6
   poly_writer->SetInput (poly_data);
+#else
+  poly_writer->SetInputData (poly_data);
+#endif
   poly_writer->SetFileName (file_name.c_str ());
 	poly_writer->SetArrayName ("Colors");
   poly_writer->Write ();
@@ -209,9 +218,12 @@ pcl::io::savePolygonFileSTL (const std::string &file_name, const pcl::PolygonMes
   vtkSmartPointer<vtkPolyData> poly_data = vtkSmartPointer<vtkPolyData>::New ();
 
   pcl::io::mesh2vtk (mesh, poly_data);
-  poly_data->Update ();
   vtkSmartPointer<vtkSTLWriter> poly_writer = vtkSmartPointer<vtkSTLWriter>::New ();
+#if VTK_MAJOR_VERSION < 6
   poly_writer->SetInput (poly_data);
+#else
+  poly_writer->SetInputData (poly_data);
+#endif
   poly_writer->SetFileName (file_name.c_str ());
   poly_writer->Write ();
 
@@ -483,9 +495,13 @@ pcl::io::saveRangeImagePlanarFilePNG (
 {
   vtkSmartPointer<vtkImageData> image = vtkSmartPointer<vtkImageData>::New();
   image->SetDimensions(range_image.width, range_image.height, 1);
+#if VTK_MAJOR_VERSION < 6
   image->SetNumberOfScalarComponents(1);
   image->SetScalarTypeToFloat();
   image->AllocateScalars();
+#else
+  image->AllocateScalars (VTK_FLOAT, 1);
+#endif
 
   int* dims = image->GetDimensions();
 
@@ -504,7 +520,11 @@ pcl::io::saveRangeImagePlanarFilePNG (
 
   vtkSmartPointer<vtkImageShiftScale> shiftScaleFilter = vtkSmartPointer<vtkImageShiftScale>::New();
   shiftScaleFilter->SetOutputScalarTypeToUnsignedChar();
+#if VTK_MAJOR_VERSION < 6
   shiftScaleFilter->SetInputConnection(image->GetProducerPort());
+#else
+  shiftScaleFilter->SetInputData (image);
+#endif
   shiftScaleFilter->SetShift(-1.0f * image->GetScalarRange()[0]); // brings the lower bound to 0
   shiftScaleFilter->SetScale(newRange/oldRange);
   shiftScaleFilter->Update();
