@@ -107,7 +107,7 @@ pcl::LineRGBD<PointXYZT, PointRGBT>::loadTemplates (const std::string &file_name
     // Search for extension
     std::string chunk_name (ltm_header.file_name);
 
-    std::transform (chunk_name.begin (), chunk_name.end (), chunk_name.begin (), tolower);
+    std::transform (chunk_name.begin (), chunk_name.end (), chunk_name.begin (), ::tolower);
     std::string::size_type it;
 
     if ((it = chunk_name.find (pcd_ext)) != std::string::npos &&
