@@ -54,19 +54,7 @@ pcl::SupervoxelClustering<PointT>::SupervoxelClustering (float voxel_resolution,
   normal_importance_(1.0f),
   label_colors_ (0)
 {
-  label_colors_.reserve (MAX_LABEL);
-  label_colors_.push_back (static_cast<uint32_t>(255) << 16 | static_cast<uint32_t>(0) << 8 | static_cast<uint32_t>(0));
-  
-  srand (static_cast<unsigned int> (time (0)));
-  for (size_t i_segment = 0; i_segment < MAX_LABEL - 1; i_segment++)
-  {
-    uint8_t r = static_cast<uint8_t>( (rand () % 256));
-    uint8_t g = static_cast<uint8_t>( (rand () % 256));
-    uint8_t b = static_cast<uint8_t>( (rand () % 256));
-    label_colors_.push_back (static_cast<uint32_t>(r) << 16 | static_cast<uint32_t>(g) << 8 | static_cast<uint32_t>(b));
-  }
-  
-  adjacency_octree_ = boost::make_shared <OctreeAdjacencyT> (resolution_);
+  adjacency_octree_.reset (new OctreeAdjacencyT (resolution_));
   if (use_single_camera_transform)
     adjacency_octree_->setTransformFunction (boost::bind (&SupervoxelClustering::transformFunction, this, _1));  
 }
@@ -80,7 +68,7 @@ pcl::SupervoxelClustering<PointT>::~SupervoxelClustering ()
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 template <typename PointT> void
-pcl::SupervoxelClustering<PointT>::setInputCloud (typename pcl::PointCloud<PointT>::ConstPtr cloud)
+pcl::SupervoxelClustering<PointT>::setInputCloud (const typename pcl::PointCloud<PointT>::ConstPtr& cloud)
 {
   if ( cloud->size () == 0 )
   {
@@ -92,6 +80,18 @@ pcl::SupervoxelClustering<PointT>::setInputCloud (typename pcl::PointCloud<Point
   adjacency_octree_->setInputCloud (cloud);
 }
 
+//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+template <typename PointT> void
+pcl::SupervoxelClustering<PointT>::setNormalCloud (typename NormalCloudT::ConstPtr normal_cloud)
+{
+  if ( normal_cloud->size () == 0 )
+  {
+    PCL_ERROR ("[pcl::SupervoxelClustering::setNormalCloud] Empty cloud set, doing nothing \n");
+    return;
+  }
+  
+  input_normals_ = normal_cloud;
+}
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 template <typename PointT> void
@@ -202,11 +202,10 @@ pcl::SupervoxelClustering<PointT>::prepareForSegmentation ()
   return true;
 }
 
-
 template <typename PointT> void
 pcl::SupervoxelClustering<PointT>::computeVoxelData ()
 {
-  voxel_centroid_cloud_ = boost::make_shared<PointCloudT> ();
+  voxel_centroid_cloud_.reset (new PointCloudT);
   voxel_centroid_cloud_->resize (adjacency_octree_->getLeafCount ());
   typename LeafVectorT::iterator leaf_itr = adjacency_octree_->begin ();
   typename PointCloudT::iterator cent_cloud_itr = voxel_centroid_cloud_->begin ();
@@ -219,40 +218,74 @@ pcl::SupervoxelClustering<PointT>::computeVoxelData ()
     new_voxel_data.idx_ = idx;
   }
   
-  
-  for (leaf_itr = adjacency_octree_->begin (); leaf_itr != adjacency_octree_->end (); ++leaf_itr)
+  //If normals were provided
+  if (input_normals_)
   {
-    VoxelData& new_voxel_data = (*leaf_itr)->getData ();
-    //For every point, get its neighbors, build an index vector, compute normal
-    std::vector<int> indices;
-    indices.reserve (81); 
-    //Push this point
-    indices.push_back (new_voxel_data.idx_);
-    for (typename LeafContainerT::iterator neighb_itr=(*leaf_itr)->begin (); neighb_itr!=(*leaf_itr)->end (); ++neighb_itr)
+    //Verify that input normal cloud size is same as input cloud size
+    assert (input_normals_->size () == input_->size ());
+    //For every point in the input cloud, find its corresponding leaf
+    typename NormalCloudT::const_iterator normal_itr = input_normals_->begin ();
+    for (typename PointCloudT::const_iterator input_itr = input_->begin (); input_itr != input_->end (); ++input_itr, ++normal_itr)
     {
-      VoxelData& neighb_voxel_data = (*neighb_itr)->getData ();
-      //Push neighbor index
-      indices.push_back (neighb_voxel_data.idx_);
-      //Get neighbors neighbors, push onto cloud
-      for (typename LeafContainerT::iterator neighb_neighb_itr=(*neighb_itr)->begin (); neighb_neighb_itr!=(*neighb_itr)->end (); ++neighb_neighb_itr)
+      //If the point is not finite we ignore it
+      if ( !pcl::isFinite<PointT> (*input_itr))
+        continue;
+      //Otherwise look up its leaf container
+        LeafContainerT* leaf = adjacency_octree_->getLeafContainerAtPoint (*input_itr);
+        
+        //Get the voxel data object
+        VoxelData& voxel_data = leaf->getData ();
+        //Add this normal in (we will normalize at the end)
+        voxel_data.normal_ += normal_itr->getNormalVector4fMap ();
+        voxel_data.curvature_ += normal_itr->curvature;
+    }
+    //Now iterate through the leaves and normalize 
+    for (leaf_itr = adjacency_octree_->begin (); leaf_itr != adjacency_octree_->end (); ++leaf_itr)
+    {
+      VoxelData& voxel_data = (*leaf_itr)->getData ();
+      voxel_data.normal_.normalize ();
+      voxel_data.owner_ = 0;
+      voxel_data.distance_ = std::numeric_limits<float>::max ();
+      //Get the number of points in this leaf
+      int num_points = (*leaf_itr)->getPointCounter ();
+      voxel_data.curvature_ /= num_points;
+    }
+  }
+  else //Otherwise just compute the normals
+  {
+    for (leaf_itr = adjacency_octree_->begin (); leaf_itr != adjacency_octree_->end (); ++leaf_itr)
+    {
+      VoxelData& new_voxel_data = (*leaf_itr)->getData ();
+      //For every point, get its neighbors, build an index vector, compute normal
+      std::vector<int> indices;
+      indices.reserve (81); 
+      //Push this point
+      indices.push_back (new_voxel_data.idx_);
+      for (typename LeafContainerT::iterator neighb_itr=(*leaf_itr)->begin (); neighb_itr!=(*leaf_itr)->end (); ++neighb_itr)
       {
-        VoxelData& neighb2_voxel_data = (*neighb_neighb_itr)->getData ();
-        indices.push_back (neighb2_voxel_data.idx_);
+        VoxelData& neighb_voxel_data = (*neighb_itr)->getData ();
+        //Push neighbor index
+        indices.push_back (neighb_voxel_data.idx_);
+        //Get neighbors neighbors, push onto cloud
+        for (typename LeafContainerT::iterator neighb_neighb_itr=(*neighb_itr)->begin (); neighb_neighb_itr!=(*neighb_itr)->end (); ++neighb_neighb_itr)
+        {
+          VoxelData& neighb2_voxel_data = (*neighb_neighb_itr)->getData ();
+          indices.push_back (neighb2_voxel_data.idx_);
+        }
       }
+      //Compute normal
+      pcl::computePointNormal (*voxel_centroid_cloud_, indices, new_voxel_data.normal_, new_voxel_data.curvature_);
+      pcl::flipNormalTowardsViewpoint (voxel_centroid_cloud_->points[new_voxel_data.idx_], 0.0f,0.0f,0.0f, new_voxel_data.normal_);
+      new_voxel_data.normal_[3] = 0.0f;
+      new_voxel_data.normal_.normalize ();
+      new_voxel_data.owner_ = 0;
+      new_voxel_data.distance_ = std::numeric_limits<float>::max ();
     }
-    //Compute normal
-    pcl::computePointNormal (*voxel_centroid_cloud_, indices, new_voxel_data.normal_, new_voxel_data.curvature_);
-    pcl::flipNormalTowardsViewpoint (voxel_centroid_cloud_->points[new_voxel_data.idx_], 0.0f,0.0f,0.0f, new_voxel_data.normal_);
-    new_voxel_data.normal_[3] = 0.0f;
-    new_voxel_data.normal_.normalize ();
-    new_voxel_data.owner_ = 0;
-    new_voxel_data.distance_ = std::numeric_limits<float>::max ();
   }
   
   
 }
 
-
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 template <typename PointT> void
 pcl::SupervoxelClustering<PointT>::expandSupervoxels ( int depth )
@@ -293,13 +326,15 @@ pcl::SupervoxelClustering<PointT>::makeSupervoxels (std::map<uint32_t,typename S
   for (typename HelperListT::iterator sv_itr = supervoxel_helpers_.begin (); sv_itr != supervoxel_helpers_.end (); ++sv_itr)
   {
     uint32_t label = sv_itr->getLabel ();
-    supervoxel_clusters[label] = boost::make_shared<Supervoxel<PointT> > ();
+    supervoxel_clusters[label].reset (new Supervoxel<PointT>);
     sv_itr->getXYZ (supervoxel_clusters[label]->centroid_.x,supervoxel_clusters[label]->centroid_.y,supervoxel_clusters[label]->centroid_.z);
     sv_itr->getRGB (supervoxel_clusters[label]->centroid_.rgba);
     sv_itr->getNormal (supervoxel_clusters[label]->normal_);
     sv_itr->getVoxels (supervoxel_clusters[label]->voxels_);
     sv_itr->getNormals (supervoxel_clusters[label]->normals_);
   }
+  //Make sure that color vector is big enough
+  initializeLabelColors ();
 }
 
 
@@ -309,7 +344,7 @@ pcl::SupervoxelClustering<PointT>::createSupervoxelHelpers (std::vector<PointT,
 {
   
   supervoxel_helpers_.clear ();
-  for (int i = 0; i < seed_points.size (); ++i)
+  for (size_t i = 0; i < seed_points.size (); ++i)
   {
     supervoxel_helpers_.push_back (new SupervoxelHelper(i+1,this));
     //Find which leaf corresponds to this seed index
@@ -351,7 +386,7 @@ pcl::SupervoxelClustering<PointT>::selectInitialSupervoxelSeeds (std::vector<Poi
   distance.resize(1,0);
   if (voxel_kdtree_ == 0)
   {
-    voxel_kdtree_ = boost::make_shared< pcl::search::KdTree<PointT> >();
+    voxel_kdtree_.reset (new pcl::search::KdTree<PointT>);
     voxel_kdtree_ ->setInputCloud (voxel_centroid_cloud_);
   }
   
@@ -368,7 +403,7 @@ pcl::SupervoxelClustering<PointT>::selectInitialSupervoxelSeeds (std::vector<Poi
   // This is number of voxels which fit in a planar slice through search volume
   // Area of planar slice / area of voxel side
   float min_points = 0.05f * (search_radius)*(search_radius) * 3.1415926536f  / (resolution_*resolution_);
-  for (int i = 0; i < seed_indices_orig.size (); ++i)
+  for (size_t i = 0; i < seed_indices_orig.size (); ++i)
   {
     int num = voxel_kdtree_->radiusSearch (seed_indices_orig[i], search_radius , neighbors, sqr_distances);
     int min_index = seed_indices_orig[i];
@@ -513,8 +548,7 @@ pcl::SupervoxelClustering<PointT>::getSupervoxelAdjacency (std::multimap<uint32_
 template <typename PointT> pcl::PointCloud<pcl::PointXYZRGBA>::Ptr
 pcl::SupervoxelClustering<PointT>::getColoredCloud () const
 {
-  
-  pcl::PointCloud<pcl::PointXYZRGBA>::Ptr colored_cloud = boost::make_shared <pcl::PointCloud<pcl::PointXYZRGBA> >();
+  pcl::PointCloud<pcl::PointXYZRGBA>::Ptr colored_cloud (new pcl::PointCloud<pcl::PointXYZRGBA>);
   pcl::copyPointCloud (*input_,*colored_cloud);
   
   pcl::PointCloud <pcl::PointXYZRGBA>::iterator i_colored;
@@ -544,7 +578,7 @@ pcl::SupervoxelClustering<PointT>::getColoredCloud () const
 template <typename PointT> pcl::PointCloud<pcl::PointXYZRGBA>::Ptr
 pcl::SupervoxelClustering<PointT>::getColoredVoxelCloud () const
 {
-  pcl::PointCloud<pcl::PointXYZRGBA>::Ptr colored_cloud = boost::make_shared< pcl::PointCloud<pcl::PointXYZRGBA> > ();
+  pcl::PointCloud<pcl::PointXYZRGBA>::Ptr colored_cloud (new pcl::PointCloud<pcl::PointXYZRGBA>);
   for (typename HelperListT::const_iterator sv_itr = supervoxel_helpers_.cbegin (); sv_itr != supervoxel_helpers_.cend (); ++sv_itr)
   {
     typename PointCloudT::Ptr voxels;
@@ -566,7 +600,7 @@ pcl::SupervoxelClustering<PointT>::getColoredVoxelCloud () const
 template <typename PointT> typename pcl::PointCloud<PointT>::Ptr
 pcl::SupervoxelClustering<PointT>::getVoxelCentroidCloud () const
 {
-  typename PointCloudT::Ptr centroid_copy = boost::make_shared<PointCloudT> ();
+  typename PointCloudT::Ptr centroid_copy (new PointCloudT);
   copyPointCloud (*voxel_centroid_cloud_, *centroid_copy);
   return centroid_copy;
 }
@@ -575,7 +609,7 @@ pcl::SupervoxelClustering<PointT>::getVoxelCentroidCloud () const
 template <typename PointT> pcl::PointCloud<pcl::PointXYZL>::Ptr
 pcl::SupervoxelClustering<PointT>::getLabeledVoxelCloud () const
 {
-  pcl::PointCloud<pcl::PointXYZL>::Ptr labeled_voxel_cloud = boost::make_shared< pcl::PointCloud<pcl::PointXYZL> > ();
+  pcl::PointCloud<pcl::PointXYZL>::Ptr labeled_voxel_cloud (new pcl::PointCloud<pcl::PointXYZL>);
   for (typename HelperListT::const_iterator sv_itr = supervoxel_helpers_.cbegin (); sv_itr != supervoxel_helpers_.cend (); ++sv_itr)
   {
     typename PointCloudT::Ptr voxels;
@@ -597,7 +631,7 @@ pcl::SupervoxelClustering<PointT>::getLabeledVoxelCloud () const
 template <typename PointT> pcl::PointCloud<pcl::PointXYZL>::Ptr
 pcl::SupervoxelClustering<PointT>::getLabeledCloud () const
 {
-  pcl::PointCloud<pcl::PointXYZL>::Ptr labeled_cloud = boost::make_shared <pcl::PointCloud<pcl::PointXYZL> >();
+  pcl::PointCloud<pcl::PointXYZL>::Ptr labeled_cloud (new pcl::PointCloud<pcl::PointXYZL>);
   pcl::copyPointCloud (*input_,*labeled_cloud);
   
   pcl::PointCloud <pcl::PointXYZL>::iterator i_labeled;
@@ -627,7 +661,7 @@ pcl::SupervoxelClustering<PointT>::getLabeledCloud () const
 template <typename PointT> pcl::PointCloud<pcl::PointNormal>::Ptr
 pcl::SupervoxelClustering<PointT>::makeSupervoxelNormalCloud (std::map<uint32_t,typename Supervoxel<PointT>::Ptr > &supervoxel_clusters)
 {
-  pcl::PointCloud<pcl::PointNormal>::Ptr normal_cloud = boost::make_shared<pcl::PointCloud<pcl::PointNormal> > ();
+  pcl::PointCloud<pcl::PointNormal>::Ptr normal_cloud (new pcl::PointCloud<pcl::PointNormal>);
   normal_cloud->resize (supervoxel_clusters.size ());
   typename std::map <uint32_t, typename pcl::Supervoxel<PointT>::Ptr>::iterator sv_itr,sv_itr_end;
   sv_itr = supervoxel_clusters.begin ();
@@ -692,6 +726,40 @@ pcl::SupervoxelClustering<PointT>::setNormalImportance (float val)
 }
 
 
+//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+template <typename PointT> void
+pcl::SupervoxelClustering<PointT>::initializeLabelColors ()
+{
+  uint32_t max_label = static_cast<uint32_t> (getMaxLabel ());
+  //If we already have enough colors, return
+  if (label_colors_.size () > max_label)
+    return;
+  
+  //Otherwise, generate new colors until we have enough
+  label_colors_.reserve (max_label + 1);
+  srand (static_cast<unsigned int> (time (0)));
+  while (label_colors_.size () <= max_label )
+  {
+    uint8_t r = static_cast<uint8_t>( (rand () % 256));
+    uint8_t g = static_cast<uint8_t>( (rand () % 256));
+    uint8_t b = static_cast<uint8_t>( (rand () % 256));
+    label_colors_.push_back (static_cast<uint32_t>(r) << 16 | static_cast<uint32_t>(g) << 8 | static_cast<uint32_t>(b));
+  }
+}
+
+//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+template <typename PointT> int
+pcl::SupervoxelClustering<PointT>::getMaxLabel () const
+{
+  int max_label = 0;
+  for (typename HelperListT::const_iterator sv_itr = supervoxel_helpers_.cbegin (); sv_itr != supervoxel_helpers_.cend (); ++sv_itr)
+  {
+    int temp = sv_itr->getLabel ();
+    if (temp > max_label)
+      max_label = temp;
+  }
+  return max_label;
+}
 
 namespace pcl
 { 
@@ -753,9 +821,30 @@ namespace pcl
       data_.xyz_[1] /= (static_cast<float> (num_points_));
       data_.xyz_[2] /= (static_cast<float> (num_points_));
     }
-    
+
+    //Explicit overloads for XYZ types
+    template<>
+    void
+    pcl::octree::OctreePointCloudAdjacencyContainer<pcl::PointXYZ,pcl::SupervoxelClustering<pcl::PointXYZ>::VoxelData>::addPoint (const pcl::PointXYZ &new_point)
+    {
+      ++num_points_;
+      //Same as before here
+      data_.xyz_[0] += new_point.x;
+      data_.xyz_[1] += new_point.y;
+      data_.xyz_[2] += new_point.z;
+    }
+
+    template<> void
+    pcl::octree::OctreePointCloudAdjacencyContainer<pcl::PointXYZ,pcl::SupervoxelClustering<pcl::PointXYZ>::VoxelData>::computeData ()
+    {
+      data_.xyz_[0] /= (static_cast<float> (num_points_));
+      data_.xyz_[1] /= (static_cast<float> (num_points_));
+      data_.xyz_[2] /= (static_cast<float> (num_points_));
+    }
+
   }
 }
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -931,36 +1020,14 @@ pcl::SupervoxelClustering<PointT>::SupervoxelHelper::updateCentroid ()
   centroid_.xyz_ = Eigen::Vector3f::Zero ();
   centroid_.rgb_ = Eigen::Vector3f::Zero ();
   typename std::set<LeafContainerT*>::iterator leaf_itr = leaves_.begin ();
-  if (leaves_.size () < 20) //Just a check to see if we have enough points to calculate a good normal
+  for ( ; leaf_itr!= leaves_.end (); ++leaf_itr)
   {
-    for ( ; leaf_itr!= leaves_.end (); ++leaf_itr)
-    {
-      const VoxelData& leaf_data = (*leaf_itr)->getData ();
-      centroid_.normal_ += leaf_data.normal_;
-      centroid_.xyz_ += leaf_data.xyz_;
-      centroid_.rgb_ += leaf_data.rgb_;
-    }
-    centroid_.normal_.normalize ();
+    const VoxelData& leaf_data = (*leaf_itr)->getData ();
+    centroid_.normal_ += leaf_data.normal_;
+    centroid_.xyz_ += leaf_data.xyz_;
+    centroid_.rgb_ += leaf_data.rgb_;
   }
-  else
-  {
-    std::vector<int> indices;
-    indices.reserve (leaves_.size ());
-    for ( ; leaf_itr!= leaves_.end (); ++leaf_itr)
-    {
-      const VoxelData& leaf_data = (*leaf_itr)->getData ();
-      centroid_.xyz_ += leaf_data.xyz_;
-      centroid_.rgb_ += leaf_data.rgb_;
-      indices.push_back (leaf_data.idx_);
-    }
-    pcl::computePointNormal (*parent_->voxel_centroid_cloud_, indices, centroid_.normal_, centroid_.curvature_);
-    PointT temp;
-    this->getXYZ (temp.x, temp.y, temp.z);
-    pcl::flipNormalTowardsViewpoint (temp, 0.0f,0.0f,0.0f, centroid_.normal_);
-    centroid_.normal_[3] = 0.0f;
-    centroid_.normal_.normalize ();
-  }
-  
+  centroid_.normal_.normalize ();
   centroid_.xyz_ /= static_cast<float> (leaves_.size ());
   centroid_.rgb_ /= static_cast<float> (leaves_.size ());
   
@@ -970,12 +1037,14 @@ pcl::SupervoxelClustering<PointT>::SupervoxelHelper::updateCentroid ()
 template <typename PointT> void
 pcl::SupervoxelClustering<PointT>::SupervoxelHelper::getVoxels (typename pcl::PointCloud<PointT>::Ptr &voxels) const
 {
-  voxels = boost::make_shared<pcl::PointCloud<PointT> > ();
+  voxels.reset (new pcl::PointCloud<PointT>);
   voxels->clear ();
   voxels->resize (leaves_.size ());
   typename pcl::PointCloud<PointT>::iterator voxel_itr = voxels->begin ();
-  typename std::set<LeafContainerT*>::iterator leaf_itr;
-  for (leaf_itr = leaves_.begin (); leaf_itr != leaves_.end (); ++leaf_itr, ++voxel_itr)
+  //typename std::set<LeafContainerT*>::iterator leaf_itr;
+  for (typename std::set<LeafContainerT*>::const_iterator leaf_itr = leaves_.begin (); 
+	  leaf_itr != leaves_.end (); 
+	  ++leaf_itr, ++voxel_itr)
   {
     const VoxelData& leaf_data = (*leaf_itr)->getData ();
     leaf_data.getPoint (*voxel_itr);
@@ -986,10 +1055,10 @@ pcl::SupervoxelClustering<PointT>::SupervoxelHelper::getVoxels (typename pcl::Po
 template <typename PointT> void
 pcl::SupervoxelClustering<PointT>::SupervoxelHelper::getNormals (typename pcl::PointCloud<Normal>::Ptr &normals) const
 {
-  normals = boost::make_shared<pcl::PointCloud<Normal> > ();
+  normals.reset (new pcl::PointCloud<Normal>);
   normals->clear ();
   normals->resize (leaves_.size ());
-  typename std::set<LeafContainerT*>::iterator leaf_itr;
+  typename std::set<LeafContainerT*>::const_iterator leaf_itr;
   typename pcl::PointCloud<Normal>::iterator normal_itr = normals->begin ();
   for (leaf_itr = leaves_.begin (); leaf_itr != leaves_.end (); ++leaf_itr, ++normal_itr)
   {
@@ -1004,7 +1073,7 @@ pcl::SupervoxelClustering<PointT>::SupervoxelHelper::getNeighborLabels (std::set
 {
   neighbor_labels.clear ();
   //For each leaf belonging to this supervoxel
-  typename std::set<LeafContainerT*>::iterator leaf_itr;
+  typename std::set<LeafContainerT*>::const_iterator leaf_itr;
   for (leaf_itr = leaves_.begin (); leaf_itr != leaves_.end (); ++leaf_itr)
   {
     //for each neighbor of the leaf
