@@ -105,6 +105,9 @@ namespace pcl
       typename pcl::PointCloud<PointT>::Ptr voxels_;
       /** \brief A Pointcloud of the normals for the points in the supervoxel */
       typename pcl::PointCloud<Normal>::Ptr normals_;
+                
+    public:
+      EIGEN_MAKE_ALIGNED_OPERATOR_NEW  
   };
   
   /** \brief Implements a supervoxel algorithm based on voxel structure, normals, and rgb values
@@ -121,7 +124,6 @@ namespace pcl
     //Forward declaration of friended helper class
     class SupervoxelHelper;
     friend class SupervoxelHelper;
-    #define MAX_LABEL 16384
     public:
       /** \brief VoxelData is a structure used for storing data within a pcl::octree::OctreePointCloudAdjacencyContainer
        *  \note It stores xyz, rgb, normal, distance, an index, and an owner.
@@ -156,6 +158,9 @@ namespace pcl
           float distance_;
           int idx_;
           SupervoxelHelper* owner_;
+          
+        public:
+          EIGEN_MAKE_ALIGNED_OPERATOR_NEW
       };
       
       typedef pcl::octree::OctreePointCloudAdjacencyContainer<PointT, VoxelData> LeafContainerT;
@@ -231,7 +236,13 @@ namespace pcl
        * \param[in] cloud The cloud to be supervoxelize
        */
       virtual void
-      setInputCloud (typename pcl::PointCloud<PointT>::ConstPtr cloud);
+      setInputCloud (const typename pcl::PointCloud<PointT>::ConstPtr& cloud);
+      
+      /** \brief This method sets the normals to be used for supervoxels (should be same size as input cloud)
+      * \param[in] normal_cloud The input normals                         
+      */
+      virtual void
+      setNormalCloud (typename NormalCloudT::ConstPtr normal_cloud);
       
       /** \brief This method refines the calculated supervoxels - may only be called after extract
        * \param[in] num_itr The number of iterations of refinement to be done (2 or 3 is usually sufficient)
@@ -246,6 +257,7 @@ namespace pcl
         * Points that belong to the same supervoxel have the same color.
         * But this function doesn't guarantee that different segments will have different
         * color(it's random). Points that are unlabeled will be black
+        * \note This will expand the label_colors_ vector so that it can accomodate all labels
         */
       typename pcl::PointCloud<PointXYZRGBA>::Ptr
       getColoredCloud () const;
@@ -266,6 +278,7 @@ namespace pcl
        * Points that belong to the same supervoxel have the same color.
        * But this function doesn't guarantee that different segments will have different
        * color(it's random). Points that are unlabeled will be black
+       * \note This will expand the label_colors_ vector so that it can accomodate all labels
        */
       pcl::PointCloud<pcl::PointXYZRGBA>::Ptr
       getColoredVoxelCloud () const;
@@ -296,8 +309,19 @@ namespace pcl
        */
       static pcl::PointCloud<pcl::PointNormal>::Ptr
       makeSupervoxelNormalCloud (std::map<uint32_t,typename Supervoxel<PointT>::Ptr > &supervoxel_clusters);
+      
+      /** \brief Returns the current maximum (highest) label */
+      int
+      getMaxLabel () const;
+      
     private:
       
+      /** \brief This method initializes the label_colors_ vector (assigns random colors to labels)
+       * \note Checks to see if it is already big enough - if so, does not reinitialize it
+       */
+      void
+      initializeLabelColors ();
+      
       /** \brief This method simply checks if it is possible to execute the segmentation algorithm with
         * the current settings. If it is possible then it returns true.
         */
@@ -355,6 +379,9 @@ namespace pcl
       /** \brief Contains the Voxelized centroid Cloud */
       typename PointCloudT::Ptr voxel_centroid_cloud_;
       
+      /** \brief Contains the Voxelized centroid Cloud */
+      typename NormalCloudT::ConstPtr input_normals_;
+      
       /** \brief Importance of color in clustering */
       float color_importance_;
       /** \brief Importance of distance from seed center in clustering */
@@ -450,16 +477,20 @@ namespace pcl
           
           size_t
           size () const { return leaves_.size (); }
-      private:
+        private:
           //Stores leaves
           std::set<LeafContainerT*> leaves_;
           uint32_t label_;
           VoxelData centroid_;
           SupervoxelClustering* parent_;
-          
-
+        public:
+          //Type VoxelData may have fixed-size Eigen objects inside
+          EIGEN_MAKE_ALIGNED_OPERATOR_NEW
       };
       
+      //Make boost::ptr_list can access the private class SupervoxelHelper
+      friend void boost::checked_delete<> (const typename pcl::SupervoxelClustering<PointT>::SupervoxelHelper *);
+      
       typedef boost::ptr_list<SupervoxelHelper> HelperListT;
       HelperListT supervoxel_helpers_;
       
